! =================================================================================================================================
! MODULE       : constantes_lmtc
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2011)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF         This module contains the standard values of the parameters for the 5 metaclasses of livestock used by ORCHIDEE.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S):
!!
!! REFERENCE(S)	:
!! - EMEP/EEA, 2019
!! - Sommer et al., 2019
!! - Paustian et al., 2006
!! - FAO, 2018
!! SVN          :
!! $HeadURL: $
!! $Date: 2020-08-21 16:22:29 +0200 (ven. 21 août 2020) $
!! $Revision: 6863 $
!! \n
!_ ================================================================================================================================

MODULE constantes_lmtc

  USE defprec
  USE constantes

  IMPLICIT NONE

  !
  ! METACLASSES CHARACTERISTICS
  !

  INTEGER(i_std), PARAMETER :: nlmc = 5                         !! Number of MTCS fixed in the code (unitless)




  CHARACTER(len=34), PARAMETER, DIMENSION(nlmc) :: LMTC_name = &  !! description of the MTC (unitless)
  & (/ 'Non dairy cattle                  ', &          !  1
  &    'Dairy cattle                      ', &          !  2
  &    'Pig                               ', &          !  3
  &    'Small ruminants                   ', &          !  4
  &    'Chicken                           '  /)         !  5

  LOGICAL, PARAMETER, DIMENSION(nlmc) :: is_ruminant_lmtc  =  &          !! flag for ruminant animal types (true/false)
  & (/ .TRUE.,   .TRUE.,   .FALSE.,   .TRUE.,   .FALSE.  /)

  REAL(r_std),PARAMETER               :: max_above_lmtc  =  0.7               !! Max aboveground biomass fraction that can be extracted (unitless)

  !
  ! ANIMAL CHARACTERISTICS
  !
  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: weight_euref_lmtc  =  &      !! European weight references  (kg)
  & (/  340.0,   600.0,   100.0,   50.0,   1.6 /)                        !! used in case of normalizing litter amount

  REAL(r_std), PARAMETER, DIMENSION(nlmc) :: xexcreted_lmtc  =  &        !! proportion of N excreted calculated from (1-Nretained)
  & (/  0.93,  0.80,  0.70,  0.90, 0.70 /)                               !! Nretained : fraction of nitrogen retained by the animal
                                                                         !! for the production of meat, milk, or wood

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: xurine_lmtc  =  &           !! Proportion of TAN in N excreted
  & (/  0.6,  0.6,  0.7,  0.5, 0.7 /)


  !
  ! ACTIVITIES CHARACTERISTICS
  !
  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: straw_euref_lmtc  =  &      !! European straw used in litter-based manure
  & (/  500.0,  1500.0,  400.0,   20.0,   0.0 /)                      !! management systems  (kg /Heads/yr)
                                                                        !! used only for Europe region?

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: n_straw_euref_lmtc  =  &     !! European N added in straw-litter
  & (/  2.0,  6.0,  1.6,   0.08,   0.0 /)                              !! (kg N /Heads/yr)

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: length_euref_lmtc  =  &     !! European length of housing (days)
  & (/  180,   180,   365, 30,  365 /)

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: xgrazing_lmtc  =  &         !! Fraction of the year where
  & (/  0.5,  0.5,  0.0,  0.92, 0.0 /)                                  !! animal graze (European)

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: xyard_lmtc  =  &            !! Fraction of the year where
  & (/  0.1, 0.25,  0.0,  0.02, 0.0/)                              !! animal is at yard (European)


  !
  ! MANURE INFORMATION
  !


  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_hou_s_lmtc  =  &      !! Emission factor of TAN during
  & (/  0.08,  0.08,  0.23,  0.22, 0.21 /)                           !! housing in solid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_hou_l_lmtc  =  &      !! Emission factor of TAN during
  & (/  0.19,  0.19,  0.27,  0.0, 0.0/)                        !! housing in liquid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_sto_l_lmtc  =  &      !! Emission factor of TAN during
  & (/  0.25,  0.25,  0.11,  0.0, 0.0/)                         !! storage in liquid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_sto_s_lmtc  =  &      !! Emission factor of TAN during
  & (/  0.32,  0.32,  0.29,  0.30, 0.19 /)                           !! storage in solid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_n2o_sto_l_lmtc  =  &   !! Emission factor of N2O during
  & (/  0.0,  0.0,  0.0,  0.0, 0.0 /)                                 !! storage in liquid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_n2o_sto_s_lmtc  =  &   !! Emission factor of N2O during
  & (/  0.02,  0.02,  0.01,  0.02, 0.02 /)                            !! storage in solid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_no_sto_l_lmtc  =  &   !! Emission factor of NO during
  & (/  0.0001,  0.0001,  0.0001,  0.0, 0.0/)                   !! storage in liquid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_no_sto_s_lmtc  =  &   !! Emission factor of NO during
  & (/  0.01,  0.01,  0.01,  0.01, 0.1 /)                            !! storage in solid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_n2_sto_l_lmtc  =  &   !! Emission factor of N2 during
  & (/  0.003,  0.003,  0.003,  0.0, 0.0/)                      !! storage in liquid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_n2_sto_s_lmtc  =  &   !! Emission factor of N2 during
  & (/  0.30,  0.30,  0.30,  0.30, 0.30 /)                           !! storage in solid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_app_l_lmtc  =  &       !! Emission factor of TAN during
  & (/  0.27,  0.27,  0.20,  0.0, 0.0/)                          !! application of liquid manure

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_app_s_lmtc  =  &       !! Emission factor of TAN during
  & (/  0.68,  0.68,  0.45,  0.90, 0.41/)                              !! application of solid manure



  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: xmin_lmtc  =  &            !! Fraction of mineralized organic N
  & (/  0.1,  0.1,  0.1,  0.1, 0.1 /)                                  !! present in manure stored

  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ximm_lmtc  =  &            !! Fraction of TAN that is immobilized in
  & (/  0.0067,  0.0067, 0.0067,  0.0067, 0.0067 /)                    !! organic matter when manure is handled
                                                                       !! as straw-based solid manure



  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_yard_lmtc  =  &       !! Emission factor of TAN when
  & (/  0.53,  0.30,  0.0,  0.75, 0.0 /)                         !! animal is at yard


  REAL(r_std),PARAMETER, DIMENSION(nlmc) :: ef_graz_lmtc  =  &       !! Emission factor of TAN during
  & (/  0.14,  0.14,  0.31,  0.09, 0.0/)                           !! grazing


END MODULE constantes_lmtc
