! =================================================================================================================================
! MODULE       : stomate_litter
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Update litter and lignine content after litter fall and
!! calculating litter decomposition.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_litter

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE function_library,    ONLY: biomass_to_lai


  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC littercalc,littercalc_clear, deadleaf

  LOGICAL, SAVE                        :: firstcall_litter = .TRUE.       !! first call
!$OMP THREADPRIVATE(firstcall_litter)

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : littercalc_clear
!!
!!\BRIEF        Set the flag ::firstcall_litter to .TRUE. and as such activate section
!! 1.1 of the subroutine littercalc (see below).
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE littercalc_clear
    firstcall_litter =.TRUE.
  END SUBROUTINE littercalc_clear


!! ================================================================================================================================
!! SUBROUTINE   : littercalc
!!
!!\BRIEF        Calculation of the litter decomposition and therefore of the
!! heterotrophic respiration from litter following Parton et al. (1987).
!!
!! DESCRIPTION  : The littercal routine splits the litter in 4 pools:
!! aboveground metaboblic, aboveground structural, belowground metabolic and
!! belowground structural. the fraction (F) of plant material going to metabolic
!! and structural is defined following Parton et al. (1987)
!! \latexonly
!! \input{littercalc1.tex}
!! \endlatexonly
!! \n
!! where L is the lignin content of the plant carbon pools considered and CN
!! its CN ratio. L and CN are fixed parameters for each plant carbon pools,
!! therefore it is the ratio between each plant carbon pool within a PFT, which
!! controlled the part of the total litter, that will be considered as
!! recalcitrant (i.e. structural litter) or labile (i.e. metabolic litter).\n
!!
!! The routine calculates the fraction of aboveground litter which is metabolic
!! or structural (the litterpart variable) which is then used in lpj_fire.f90.\n
!!
!! In the section 2, the routine calculate the new plant material entering the
!! litter pools by phenological death of plants organs (corresponding to the
!! variable turnover) and by fire, herbivory and others non phenological causes
!! (variable bm_to_litter). This calculation is first done for each PFT and then
!! the values calculated for each PFT are added up. Following the same approach
!! the lignin content of the total structural litter is calculated and will be
!! then used as a factor control of the decomposition of the structural litter
!! (lignin_struc) in the section 5.1.2. A test is performed to avoid that we add
!! more lignin than structural litter. Finally, the variable litterpart is
!! updated.\n
!!
!! In the section 3 and 4 the temperature and the moisture controlling the
!! decomposition are calculated for above and belowground. For aboveground
!! litter, air temperature and litter moisture are calculated in sechiba and used
!! directly. For belowground, soil temperature and moisture are also calculated
!! in sechiba but are modulated as a function of the soil depth. The modulation
!! is a multiplying factor exponentially distributed between 0 (in depth) and 1
!! in surface.\n
!!
!! Then, in the section 5, the routine calculates the structural litter decomposition
!! (C) following first order kinetics following Parton et al. (1987).
!! \latexonly
!! \input{littercalc2.tex}
!! \endlatexonly
!! \n
!! with k the decomposition rate of the structural litter.
!! k corresponds to
!! \latexonly
!! \input{littercalc3.tex}
!! \endlatexonly
!! \n
!! with littertau the turnover rate, T a function of the temperature and M a function of
!! the moisture described below.\n
!!
!! Then, the fraction of dead leaves (DL) composed by aboveground structural litter is
!! calculated as following
!! \latexonly
!! \input{littercalc4.tex}
!! \endlatexonly
!! \n
!! with k the decomposition rate of the structural litter previously
!! described.\n
!!
!! In the section 5.1, the fraction of decomposed structural litter
!! incorporated to the soil (Input) and its associated heterotrophic respiration are
!! calculated. For structural litter, the C decomposed could go in the active
!! soil carbon pool or in the slow carbon, as described in
!! stomate_soilcarbon.f90.\n
!! \latexonly
!! \input{littercalc5.tex}
!! \endlatexonly
!! \n
!! with f a parameter describing the fraction of structural litter incorporated
!! into the considered soil carbon pool, C the amount of litter decomposed and L
!! the amount of lignin in the litter. The litter decomposed which is not
!! incorporated into the soil is respired.\n
!!
!! In the section 5.2, the fraction of decomposed metabolic litter
!! incorporated to the soil and its associated heterotrophic respiration are
!! calculated with the same approaches presented for 5.1 but no control factor
!! depending on the lignin content are used.\n
!!
!! In the section 6 the dead leaf cover is calculated through a call to the
!! deadleaf subroutine presented below.\n
!!
!! In the section 7, if the flag SPINUP_ANALYTIC is set to true, we fill MatrixA
!! and VectorB following Lardy(2011).
!!
!! MAIN OUTPUT VARIABLES: ::deadleaf_cover, ::resp_hetero_litter
!! ::control_temp, ::control_moist
!!
!! REFERENCES:
!! - Parton, WJ, Schimel, DS, Cole, CV, and Ojima, DS. 1987. Analysis
!! of factors controlling soil organic matter levels in Great Plains
!! grasslands. Soil Science Society of America journal (USA)
!! (51):1173-1179.
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    :
!! \latexonly
!! \includegraphics(scale=0.5){littercalcflow.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE littercalc (npts, &
       turnover, bm_to_litter, tree_bm_to_litter,&
       veget_cov_max, tsurf, tsoil, soilhum, litterhum, soil_n_min, &
       input, harvest_above, c_litter_manure, litter, dead_leaves, lignin_struc, &
       lignin_wood, n_mineralisation, deadleaf_cover, resp_hetero_litter, &
       som_input, control_temp, control_moist, &
       MatrixA, VectorB, CN_target, CN_som_litter_longterm, tau_CN_longterm, &
       tsoil_decomp)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                  :: npts               !! Domain size - number of grid pixels
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: turnover         !! Turnover rates of plant biomass
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: bm_to_litter     !! Conversion of biomass to litter
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: tree_bm_to_litter!! Conversion of biomass to litter
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                  :: veget_cov_max      !! PFT "Maximal" coverage fraction of a PFT
                                                                                      !! defined in the input vegetation map
                                                                                      !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: tsurf              !! Temperature (K) at the surface
    REAL(r_std), DIMENSION(npts,nslm), INTENT(in)               :: tsoil              !! Soil temperature (K)
    REAL(r_std), DIMENSION(npts,nslm), INTENT(in)               :: soilhum            !! Daily soil humidity of each soil layer
                                                                                      !! (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: litterhum          !! Daily litter humidity (unitless)
    REAL(r_std), DIMENSION(npts,nvm,ninput), INTENT(in)         :: input              !! nitrogen inputs into the soil  (gN/m**2/day)

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts), INTENT(out)                   :: deadleaf_cover     !! Fraction of soil covered by dead leaves
                                                                                      !! over all PFTs (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)               :: resp_hetero_litter !! Litter heterotrophic respiration. The unit
                                                                                      !! is given by m^2 of ground.
                                                                                      !! @tex $(gC dt_sechiba one\_day^{-1}) m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(out) :: som_input        !! Quantity of Carbon (or Nitrogen) going into SOM pools
                                                                                      !! from litter decomposition. The unit is
                                                                                      !! given by m^2 of ground
                                                                                      !! @tex $(gC(orN) m^{-2} dt\_slow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)             :: control_temp       !! Temperature control of heterotrophic
                                                                                      !! respiration, above and below (0-1,
                                                                                      !! unitless)
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)             :: control_moist      !! Moisture control of heterotrophic
                                                                                      !! respiration (0.25-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(out) :: MatrixA          !! Matrix containing the fluxes between the
                                                                                      !! carbon pools per sechiba time step
                                                                                      !! @tex $(gC.m^2.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out)         :: VectorB          !! Vector containing the litter increase per
                                                                                      !! sechiba time step
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncarb), INTENT(out)           :: CN_target        !! C to N ratio of SOM flux from one pool to another (gN m-2 dt-1)
    REAL(r_std), DIMENSION(npts), INTENT(out)                     :: tsoil_decomp     !! Temperature used for decompostition in soil (K)


    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(inout) :: litter   !! Metabolic and structural litter,above and
                                                                                      !! below ground. The unit is given by m^2 of
                                                                                      !! ground @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)       :: dead_leaves        !! Dead leaves per ground unit area, per PFT,
                                                                                      !! metabolic and structural in
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)       :: lignin_struc       !! Ratio Lignin content in structural litter,
                                                                                      !! above and below ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)       :: lignin_wood        !! Ratio Lignin/Carbon in woody litter,
                                                                                      !! above and below ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)             :: n_mineralisation   !! Mineral N pool (gN m-2)

    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(inout)     :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), INTENT(inout)                              :: tau_CN_longterm        !! Counter used for calculating the longterm CN_ratio of som and litter pools [seconds]
    REAL(r_std), DIMENSION(npts,nelements), INTENT(inout)       :: harvest_above      !! Harvest above ground biomass for
                                                                                      !! agriculture @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)       :: c_litter_manure      !! Carbon manure returned in the soil as litter
                                                                                        !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nnspec),INTENT(inout)          :: soil_n_min         !! mineral nitrogen in the soil (gN/m**2)

    !! 0.4 Local variables

    REAL(r_std)                                                 :: dt                 !! Number of sechiba(fast processes) time-step per day
    REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:)         :: litterfrac         !! The fraction of leaves, wood, etc. that
                                                                                      !! goes into metabolic and structural
                                                                                      !! litterpools (0-1, unitless)
!$OMP THREADPRIVATE(litterfrac)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)                :: z_soil             !! Soil levels (m)
!$OMP THREADPRIVATE(z_soil)
    REAL(r_std), DIMENSION(npts)                                :: rpc                !! Integration constant for vertical root
                                                                                      !! profiles (unitless)
    REAL(r_std), SAVE, DIMENSION(nlitt)                         :: litter_turn        !! Turnover time in litter pools (days)
!$OMP THREADPRIVATE(litter_turn)
    REAL(r_std), SAVE, DIMENSION(nlitt,ncarb,nlevs)             :: frac_soil          !! Fraction of litter that goes into soil
                                                                                      !! (litter -> carbon, above and below). The
                                                                                      !! remaining part goes to the atmosphere
!$OMP THREADPRIVATE(frac_soil)
    REAL(r_std), DIMENSION(npts)                                :: soilhum_decomp     !! Humidity used for decompostition in soil
                                                                                      !! (unitless)
    REAL(r_std), DIMENSION(npts)                                :: fd                 !! Fraction of structural or metabolic litter
                                                                                      !! decomposed (unitless)
    REAL(r_std), DIMENSION(npts,nelements)                      :: qd                 !! Quantity of structural or metabolic litter
                                                                                      !! decomposed @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: old_struc          !! Old structural litter, above and below
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: old_woody          !! Old woody litter, above and below
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements)      :: litter_inc         !! Increase of metabolic and structural
                                                                                      !! litter, above and below ground. The unit
                                                                                      !! is given by m^2 of ground.
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: lignin_struc_inc   !! Lignin increase in structural litter,
                                                                                      !! above and below ground. The unit is given
                                                                                      !! by m^2 of ground.
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: lignin_wood_inc    !! Lignin increase in woody litter,
                                                                                      !! above and below ground. The unit is given
                                                                                      !! by m^2 of ground.
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts)                                :: zdiff_min          !! Intermediate field for looking for minimum
                                                                                      !! of what? this is not used in the code.
                                                                                      !! [??CHECK] could we delete it?
    CHARACTER(LEN=10), DIMENSION(nlitt)                         :: litter_str         !! Messages to write output information about
                                                                                      !! the litter
    CHARACTER(LEN=22), DIMENSION(nparts)                        :: part_str           !! Messages to write output information about
                                                                                      !! the plant
    CHARACTER(LEN=7), DIMENSION(ncarb)                          :: carbon_str         !! Messages to write output information about
                                                                                      !! the soil carbon
    CHARACTER(LEN=5), DIMENSION(nlevs)                          :: level_str          !! Messages to write output information about
                                                                                      !! the level (aboveground or belowground litter)
    INTEGER(i_std)                                              :: i,j,k,l,m          !! Indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_soil_n_min       !! soil_n_min function response used for defing C to N target ratios (-)
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_pnc              !! plant nitrogen concentration function response used for defining C to N target ratios (-)
    INTEGER(i_std)                                              :: itarget            !! target som pool
    REAL(r_std), DIMENSION(npts,nvm,nparts)                     :: CN                 !! CN ratio of the litter pools
    REAL(r_std), DIMENSION(npts,nelements)                      :: summ
    INTEGER(i_std)                                              :: ier                !! Error handling
    REAL(r_std), DIMENSION(nlitt)                               :: tree_litterfrac
!_ ================================================================================================================================

    IF (printlev>=3) WRITE(numout,*) 'Entering littercalc'

  !! 1. Initialisations of the different fields during the first call of the routine
    dt = dt_sechiba/one_day
    IF ( firstcall_litter ) THEN


       IF ( .NOT. ALLOCATED(litterfrac) ) THEN
          ALLOCATE(litterfrac(npts,nvm,nparts,nlitt))
       ENDIF


       !! 1.1.4 residence times in litter pools (days)
       litter_turn(imetabolic) = turn_metabolic / one_year      ! .5 years
       litter_turn(istructural) = turn_struct / one_year     ! 3 years
       litter_turn(iwoody) = turn_woody / one_year    !!!!???? 30 years (2.45)

       !! 1.1.5 decomposition flux fraction that goes into soil
       !       (litter -> carbon, above and below)
       !       1-frac_soil goes into atmosphere
       frac_soil(:,:,:) = zero

       ! structural litter: lignin fraction goes into slow pool + respiration,
       !                    rest into active pool + respiration
       frac_soil(istructural,isurface,iabove) = frac_soil_struct_sua
       frac_soil(istructural,iactive,ibelow) = frac_soil_struct_ab
       frac_soil(istructural,islow,iabove) = frac_soil_struct_sa
       frac_soil(istructural,islow,ibelow) = frac_soil_struct_sb

       ! metabolic litter: all goes into active pool + respiration.
       !   Nothing into slow or passive pool.
       frac_soil(imetabolic,isurface,iabove) = frac_soil_metab_sua
       frac_soil(imetabolic,iactive,ibelow) = frac_soil_metab_ab


       !! 1.2 soil levels
       ALLOCATE(z_soil(0:nslm), stat=ier)
       IF ( ier /= 0 ) CALL ipslerr_p(3,'littercalc','Pb in allocate of z_soil','','')

       z_soil(0) = zero
       z_soil(1:nslm) = zlt(1:nslm)


       !! 1.3 messages
       litter_str(imetabolic) = 'metabolic'
       litter_str(istructural) = 'structural'
       litter_str(iwoody) = 'woody'

       carbon_str(iactive) = 'active'
       carbon_str(isurface) = 'surface'
       carbon_str(islow) = 'slow'
       carbon_str(ipassive) = 'passive'

       level_str(iabove) = 'above'
       level_str(ibelow) = 'below'

       part_str(ileaf) = 'leaves'
       part_str(isapabove) = 'sap above ground'
       part_str(isapbelow) = 'sap below ground'
       part_str(iheartabove) = 'heartwood above ground'
       part_str(iheartbelow) = 'heartwood below ground'
       part_str(iroot) = 'roots'
       part_str(ifruit) = 'fruits'
       part_str(icarbres) = 'carbohydrate reserve'
       part_str(ilabile) = 'labile reserve'


       IF (printlev >=2) THEN
          WRITE(numout,*) 'litter:'
          WRITE(numout,*) '   > C/N ratios: '
          DO k = 1, nparts
             WRITE(numout,*) '       ', part_str(k), ': ',CN_fix(k)
          ENDDO

          WRITE(numout,*) '   > Lignine/C ratios: '
          DO k = 1, nparts
             WRITE(numout,*) '       ', part_str(k), ': ',LC(:,k)
          ENDDO

          WRITE(numout,*) '   > scaling depth for decomposition (m): ',z_decomp

          WRITE(numout,*) '   > minimal carbon residence time in litter pools (d):'
          DO m = 1, nlitt
             WRITE(numout,*) '       ',litter_str(m),':',litter_turn(m)
          ENDDO

          WRITE(numout,*) '   > litter decomposition flux fraction that really goes '
          WRITE(numout,*) '     into carbon pools (rest into the atmosphere):'
          DO m = 1, nlitt
             DO l = 1, nlevs
                DO k = 1, ncarb
                   WRITE(numout,*) '       ',litter_str(m),' ',level_str(l),' -> ',&
                        carbon_str(k),':', frac_soil(m,k,l)
                ENDDO
             ENDDO
          ENDDO
       END IF ! printlev >=2
       firstcall_litter = .FALSE.

    ENDIF


    !! 1.1.3 litter fractions:
    !!   what fraction of leaves, wood, etc. goes into metabolic and structural litterpools

    ! PFT 1 needs to be initialised for when everything is printed below
    litterfrac(:,1,:,:)=zero
    CN(:,:,:) = zero

    DO k = 1, nparts
       WHERE((bm_to_litter(:,:,k,initrogen)+turnover(:,:,k,initrogen)).GT.min_stomate)
          CN(:,:,k) = (bm_to_litter(:,:,k,icarbon)+turnover(:,:,k,icarbon))/ &
               (bm_to_litter(:,:,k,initrogen)+turnover(:,:,k,initrogen))
       ELSEWHERE
          CN(:,:,k) = CN_fix(k)
       ENDWHERE
       DO j = 2,nvm

          IF ( ((k == isapabove) .OR. (k == isapbelow) .OR. (k == iheartabove) .OR. (k == iheartbelow)) &
               .AND. is_tree(j) ) THEN
             litterfrac(:,j,k,iwoody) = 1.
             litterfrac(:,j,k,imetabolic) = zero
             litterfrac(:,j,k,istructural) = zero
          ELSE
             IF( (k == icarbres) .OR. (k == ilabile)) THEN
                litterfrac(:,j,k,imetabolic) = 1.0
             ELSE
                litterfrac(:,j,k,imetabolic) = MAX(metabolic_ref_frac - metabolic_LN_ratio  * LC(j,k) * CN(:,j,k),zero)
             ENDIF
             litterfrac(:,j,k,istructural) = 1. - litterfrac(:,j,k,imetabolic)
             litterfrac(:,j,k,iwoody) = 0.0

          ENDIF

       END DO

    END DO

    tree_litterfrac(iwoody)=1.
    tree_litterfrac(imetabolic) = zero
    tree_litterfrac(istructural) = zero


    CN_target(:,:,:)=0.0

    ! C to N target ratios of differnt pools
    f_soil_n_min = MIN((soil_n_min(:,:,iammonium) + soil_n_min(:,:,initrate)), 2.)
    ! CN ratio ranges between 3 and 15 for active pool
    ! Figure 4 of Parton et al. (1993) show lower CN values for active pool of 2 rather than 3
    CN_target(:,:,iactive)= CN_target_iactive_ref + CN_target_iactive_Nmin * f_soil_n_min(:,:)
    ! CN ratio ranges between 12 and 20 for slow pool
    CN_target(:,:,islow)= CN_target_islow_ref + CN_target_islow_Nmin * f_soil_n_min(:,:)
    ! CN ratio ranges between 7 and 10 for passive pool
    ! OCN uses a fixed value of 9. (don't know why)
    ! Figure 4 of Parton et al. (1993) show lower CN values for passive pool of 3 rather than 7
    CN_target(:,:,ipassive)= CN_target_ipassive_ref + CN_target_ipassive_Nmin * f_soil_n_min(:,:)


    ! plant nitrogen content (%)
    WHERE(litter(:,imetabolic,:,iabove,icarbon)+litter(:,istructural,:,iabove,icarbon).GT.0.)
       f_pnc(:,:)=MIN((litter(:,imetabolic,:,iabove,initrogen)+litter(:,istructural,:,iabove,initrogen)) / &
            (2.*(litter(:,imetabolic,:,iabove,icarbon)+litter(:,istructural,:,iabove,icarbon))) * 100., 2.)
    ELSEWHERE
       f_pnc(:,:)=0.
    ENDWHERE

    CN_target(:,:,isurface) = CN_target_isurface_ref + CN_target_isurface_pnc * f_pnc(:,:)



    !! 1.4 set output to zero
    deadleaf_cover(:) = zero
    resp_hetero_litter(:,:) = zero
    som_input(:,:,:,:) = zero


  !! 2. Add biomass to different litterpools (per m^2 of ground)

    !! 2.1 first, save old structural litter (needed for lignin fractions).
    !     above/below
    DO l = 1, nlevs !Loop over litter levels (above and below ground)
       DO m = 1,nvm !Loop over PFTs

          old_struc(:,m,l) = litter(:,istructural,m,l,icarbon)
          old_woody(:,m,l) = litter(:,iwoody,m,l,icarbon)
       ENDDO
    ENDDO


    !! 2.2 update litter, dead leaves, and lignin content in structural litter
    litter_inc(:,:,:,:,:) = zero
    lignin_struc_inc(:,:,:) = zero
    lignin_wood_inc(:,:,:) = zero

    DO j = 1,nvm !Loop over PFTs

       !! 2.2.1 litter
       DO k = 1, nlitt    !Loop over litter pools (metabolic and structural)

          DO l = 1, nelements  ! Loop over element pools (carbon and nitrogen)
          !! 2.2.2 calculate litter increase (per m^2 of ground).
          !       Only a given fracion of fruit turnover is directly coverted into litter.
          !       Litter increase for each PFT, structural and metabolic, above/below
             litter_inc(:,k,j,iabove,l) = &
                  litterfrac(:,j,ileaf,k) * bm_to_litter(:,j,ileaf,l) + &
                  litterfrac(:,j,isapabove,k) * (bm_to_litter(:,j,isapabove,l)-tree_bm_to_litter(:,j,isapabove,l)) + &
                  litterfrac(:,j,iheartabove,k) * (bm_to_litter(:,j,iheartabove,l)-tree_bm_to_litter(:,j,iheartabove,l)) + &
                  tree_litterfrac(k) * tree_bm_to_litter(:,j,isapabove,l) + &
                  tree_litterfrac(k) * tree_bm_to_litter(:,j,iheartabove,l) + &
                  litterfrac(:,j,ifruit,k) * bm_to_litter(:,j,ifruit,l) + &
                  litterfrac(:,j,icarbres,k) * bm_to_litter(:,j,icarbres,l) + &
                  litterfrac(:,j,ilabile,k) * bm_to_litter(:,j,ilabile,l) + &
                  litterfrac(:,j,ileaf,k) * turnover(:,j,ileaf,l) + &
                  litterfrac(:,j,isapabove,k) * turnover(:,j,isapabove,l) + &
                  litterfrac(:,j,iheartabove,k) * turnover(:,j,iheartabove,l) + &
                  litterfrac(:,j,ifruit,k) * turnover(:,j,ifruit,l) + &
                  litterfrac(:,j,icarbres,k) * turnover(:,j,icarbres,l)+ &
                  litterfrac(:,j,ilabile,k) * turnover(:,j,ilabile,l)

             litter_inc(:,k,j,ibelow,l) = &
                  litterfrac(:,j,isapbelow,k) * (bm_to_litter(:,j,isapbelow,l)-tree_bm_to_litter(:,j,isapbelow,l)) + &
                  litterfrac(:,j,iheartbelow,k) * (bm_to_litter(:,j,iheartbelow,l)-tree_bm_to_litter(:,j,iheartbelow,l)) + &
                  tree_litterfrac(k) * tree_bm_to_litter(:,j,isapbelow,l) + &
                  tree_litterfrac(k) * tree_bm_to_litter(:,j,iheartbelow,l) + &
                  litterfrac(:,j,iroot,k) * bm_to_litter(:,j,iroot,l) + &
                  litterfrac(:,j,isapbelow,k) * turnover(:,j,isapbelow,l) + &
                  litterfrac(:,j,iheartbelow,k) * turnover(:,j,iheartbelow,l) + &
                  litterfrac(:,j,iroot,k) * turnover(:,j,iroot,l)

          ENDDO
          !! 2.2.3 dead leaves, for soil cover.
          dead_leaves(:,j,k) = &
               dead_leaves(:,j,k) + &
               litterfrac(:,j,ileaf,k) * ( bm_to_litter(:,j,ileaf,icarbon) + turnover(:,j,ileaf,icarbon) )
       ENDDO


       !! 2.2.4 lignin increase in structural litter
       lignin_struc_inc(:,j,iabove) = &
            LC(j,ileaf) * bm_to_litter(:,j,ileaf,icarbon) + &
            LC(j,ifruit) * bm_to_litter(:,j,ifruit,icarbon) + &
            LC(j,icarbres) * bm_to_litter(:,j,icarbres,icarbon) + &
            LC(j,ilabile) * bm_to_litter(:,j,ilabile,icarbon) + &
            LC(j,ileaf) * turnover(:,j,ileaf,icarbon) + &
            LC(j,ifruit) * turnover(:,j,ifruit,icarbon) + &
            LC(j,icarbres) * turnover(:,j,icarbres,icarbon) + &
            LC(j,ilabile) * turnover(:,j,ilabile,icarbon)

       lignin_struc_inc(:,j,ibelow) = &
            LC(j,iroot) * bm_to_litter(:,j,iroot,icarbon) + &
            LC(j,iroot)*turnover(:,j,iroot,icarbon)


       IF(is_tree(j)) THEN
          !! 2.2.4 lignin increase in woody litter

          lignin_wood_inc(:,j,iabove) = &
               LC(j,isapabove) * turnover(:,j,isapabove,icarbon) + &
               LC(j,iheartabove) * turnover(:,j,iheartabove,icarbon)

          lignin_wood_inc(:,j,ibelow) = &
               LC(j,isapbelow)*turnover(:,j,isapbelow,icarbon) + &
               LC(j,iheartbelow)*turnover(:,j,iheartbelow,icarbon)
       ELSE
          !! 2.2.4 lignin increase in structural litter
          lignin_struc_inc(:,j,iabove) = lignin_struc_inc(:,j,iabove) + &
               LC(j,isapabove) * turnover(:,j,isapabove,icarbon) + &
               LC(j,iheartabove) * turnover(:,j,iheartabove,icarbon)

          lignin_struc_inc(:,j,ibelow) = lignin_struc_inc(:,j,ibelow) + &
               LC(j,isapbelow)*turnover(:,j,isapbelow,icarbon) + &
               LC(j,iheartbelow)*turnover(:,j,iheartbelow,icarbon)
       ENDIF


       lignin_wood_inc(:,j,iabove) = lignin_wood_inc(:,j,iabove)+&
            LC(j,isapabove) * tree_bm_to_litter(:,j,isapabove,icarbon) + &
            LC(j,iheartabove) * tree_bm_to_litter(:,j,iheartabove,icarbon)

       lignin_wood_inc(:,j,ibelow) = lignin_wood_inc(:,j,ibelow)+ &
            LC(j,isapbelow) * tree_bm_to_litter(:,j,isapbelow,icarbon) + &
            LC(j,iheartbelow) * tree_bm_to_litter(:,j,iheartbelow,icarbon)

       !! 2.2.4 lignin increase in structural litter
       lignin_struc_inc(:,j,iabove) = lignin_struc_inc(:,j,iabove) + &
            LC(j,isapabove) * (bm_to_litter(:,j,isapabove,icarbon)-tree_bm_to_litter(:,j,isapabove,icarbon)) + &
            LC(j,iheartabove) * (bm_to_litter(:,j,iheartabove,icarbon)-tree_bm_to_litter(:,j,iheartabove,icarbon))

       lignin_struc_inc(:,j,ibelow) = lignin_struc_inc(:,j,ibelow) + &
            LC(j,isapbelow) * (bm_to_litter(:,j,isapbelow,icarbon)-tree_bm_to_litter(:,j,isapbelow,icarbon)) + &
            LC(j,iheartbelow) * (bm_to_litter(:,j,iheartbelow,icarbon)-bm_to_litter(:,j,iheartbelow,icarbon))

    ENDDO

    !! 2.2.5 add new litter (struct/met, above/below)
    litter(:,:,:,:,:) = litter(:,:,:,:,:) + litter_inc(:,:,:,:,:)

    !! 2.2.6 for security: can't add more lignin than structural litter (above/below)
    DO l = 1, nlevs !Loop over litter levels (above and below ground)
       DO m = 1,nvm !Lopp over PFTs

          lignin_struc_inc(:,m,l) = &
               MIN( lignin_struc_inc(:,m,l), litter_inc(:,istructural,m,l,icarbon) )
          lignin_wood_inc(:,m,l) = &
               MIN( lignin_wood_inc(:,m,l), litter_inc(:,iwoody,m,l,icarbon) )

       ENDDO
    ENDDO


    !! 2.2.7 new lignin content: add old lignin and lignin increase, divide by
    !!       total structural litter (above/below)

    WHERE ( litter(:,istructural,:,:,icarbon) .GT. min_stomate )
       lignin_struc(:,:,:) =  &
            ( lignin_struc(:,:,:)*old_struc(:,:,:) + lignin_struc_inc(:,:,:) ) / &
            litter(:,istructural,:,:,icarbon)
    ELSEWHERE
       lignin_struc(:,:,:) = zero
    ENDWHERE

    WHERE ( litter(:,iwoody,:,:,icarbon) .GT. min_stomate )
       lignin_wood(:,:,:) =  &
            ( lignin_wood(:,:,:)*old_woody(:,:,:) + lignin_wood_inc(:,:,:) ) / &
            litter(:,iwoody,:,:,icarbon)
    ELSEWHERE
       lignin_wood(:,:,:) = zero
    ENDWHERE

    !! 2.2.8 Add the manure into the metabolic above ground pool if enough C is
    !! harvested over the pixel. If not enough C is harvested a part goes as
    !! litter and the other part goes at mineral N. Here we assume that N is not
    !! limiting, only the C harvested is limiting

    c_litter_manure(:,:) = zero
    DO m=1,nvm
      
      IF(.NOT. ok_livestock_emissions) THEN
       WHERE((veget_cov_max(:,m).GT.min_stomate) .AND. (harvest_above(:,icarbon)*dt .GE. input(:,m,imanure)*cn_ratio_manure*dt))

          litter(:,imetabolic,m,iabove,icarbon) = litter(:,imetabolic,m,iabove,icarbon) + &
               input(:,m,imanure)*cn_ratio_manure*dt
          litter(:,imetabolic,m,iabove,initrogen) = litter(:,imetabolic,m,iabove,initrogen) + &
               input(:,m,imanure)*dt

          harvest_above(:,icarbon) = harvest_above(:,icarbon) - input(:,m,imanure)*cn_ratio_manure

       ELSEWHERE((veget_cov_max(:,m).GT.min_stomate) .AND. (harvest_above(:,icarbon)*dt .LT. input(:,m,imanure)*cn_ratio_manure*dt))

          litter(:,imetabolic,m,iabove,icarbon) = litter(:,imetabolic,m,iabove,icarbon) + &
               harvest_above(:,icarbon)*dt
          litter(:,imetabolic,m,iabove,initrogen) = litter(:,imetabolic,m,iabove,initrogen) + &
               harvest_above(:,icarbon)/cn_ratio_manure*dt

          soil_n_min(:,m,iammonium) = soil_n_min(:,m,iammonium) &
               + (input(:,m,imanure)-harvest_above(:,icarbon)/cn_ratio_manure)*ratio_nh4_fert*dt
          soil_n_min(:,m,initrate) = soil_n_min(:,m,initrate) &
               + (input(:,m,imanure)-harvest_above(:,icarbon)/cn_ratio_manure)*(1.-ratio_nh4_fert)*dt

          harvest_above(:,icarbon) = zero

       ENDWHERE

      END IF
    ENDDO



    !! 3. Temperature control on decay: Factor between 0 and 1

    !! 3.1 above: surface temperature
    control_temp(:,iabove) = control_temp_func (npts, tsurf)


    !! 3.2 below: convolution of temperature and decomposer profiles
    !!            (exponential decomposer profile supposed)

    !! 3.2.1 rpc is an integration constant such that the integral of the root profile is 1.
    rpc(:) = un / ( un - EXP( -z_soil(nslm) / z_decomp ) )

    !! 3.2.2 integrate over the nslm levels
    tsoil_decomp(:) = zero

    DO l = 1, nslm

       tsoil_decomp(:) = &
            tsoil_decomp(:) + tsoil(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_decomp ) - EXP( -z_soil(l)/z_decomp ) )

    ENDDO

    control_temp(:,ibelow) = control_temp_func (npts, tsoil_decomp)

  !! 4. Moisture control. Factor between 0 and 1

    !! 4.1 above the ground: litter humidity
    control_moist(:,iabove) = control_moist_func (npts, litterhum)

    !
    !! 4.2 below: convolution of humidity and decomposer profiles
    !            (exponential decomposer profile supposed)

    !! 4.2.1 rpc is an integration constant such that the integral of the root profile is 1.
    rpc(:) = un / ( un - EXP( -z_soil(nslm) / z_decomp ) )

    !! 4.2.2 integrate over the nslm levels
    soilhum_decomp(:) = zero

    DO l = 1, nslm !Loop over soil levels

       soilhum_decomp(:) = &
            soilhum_decomp(:) + soilhum(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_decomp ) - EXP( -z_soil(l)/z_decomp ) )

    ENDDO

    control_moist(:,ibelow) = control_moist_func (npts, soilhum_decomp)

  !! 5. fluxes from litter to carbon pools and respiration

    DO l = 1, nlevs !Loop over litter levels (above and below ground)
       DO m = 1,nvm !Loop over PFTs

    IF ( ok_soil_carbon_discretization ) THEN
          itarget=iactive
    ELSE
          IF (l.EQ.iabove)THEN
             itarget=isurface
          ELSE
             itarget=iactive
          ENDIF
    ENDIF
          !! 5.1 structural litter: goes into active and slow carbon pools + respiration

          !! 5.1.1 total quantity of structural litter which is decomposed
          fd(:) = dt*litter_turn(istructural) * &
               control_temp(:,l) * control_moist(:,l) * exp( -litter_struct_coef * lignin_struc(:,m,l) )

          DO k = 1,nelements

             qd(:,k) = litter(:,istructural,m,l,k) * fd(:)

          END DO

          litter(:,istructural,m,l,:) = litter(:,istructural,m,l,:) - qd(:,:)
          n_mineralisation(:,m) = n_mineralisation(:,m) + qd(:,initrogen)

          !! 5.1.2 decompose same fraction of structural part of dead leaves. Not exact
          !!       as lignine content is not the same as that of the total structural litter.
          ! to avoid a multiple (for ibelow and iabove) modification of dead_leaves,
          ! we do this test to do this calcul only ones in 1,nlev loop
          if (l == iabove)  dead_leaves(:,m,istructural) = dead_leaves(:,m,istructural) * ( un - fd(:) )

          !! 5.1.3 non-lignin fraction of structural litter goes into
          !!       active (or surface) carbon pool + respiration
          som_input(:,itarget,m,icarbon) = som_input(:,itarget,m,icarbon) + &
               frac_soil(istructural,itarget,l) * qd(:,icarbon) * ( 1. - lignin_struc(:,m,l) ) / dt

          !BE CAREFUL: Here resp_hetero_litter is divided by dt to have a value which corresponds to
          ! the sechiba time step but then in stomate.f90 resp_hetero_litter is multiplied by dt.
          ! Perhaps it could be simplified. Moreover, we must totally adapt the routines to the dtradia/one_day
          ! time step and avoid some constructions that could create bug during future developments.
          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
               ( 1. - frac_soil(istructural,itarget,l) ) * qd(:,icarbon) * &
               ( 1. - lignin_struc(:,m,l) ) / dt

          !! 5.1.4 lignin fraction of structural litter goes into
          !!       slow carbon pool + respiration
          som_input(:,islow,m,icarbon) = som_input(:,islow,m,icarbon) + &
               frac_soil(istructural,islow,l) * qd(:,icarbon) * lignin_struc(:,m,l) / dt

      !BE CAREFUL: Here resp_hetero_litter is divided by dt to have a value which corresponds to
      ! the sechiba time step but then in stomate.f90 resp_hetero_litter is multiplied by dt.
      ! Perhaps it could be simplified. Moreover, we must totally adapt the routines to the dt_sechiba/one_day
      ! time step and avoid some constructions that could create bug during future developments.
          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
               ( 1. - frac_soil(istructural,islow,l) ) * qd(:,icarbon) * lignin_struc(:,m,l) / dt


          !! 5.2 metabolic litter goes into active carbon pool + respiration

          !! 5.2.1 total quantity of metabolic litter that is decomposed
          fd(:) = dt*litter_turn(imetabolic) * control_temp(:,l) * control_moist(:,l)

          DO k = 1,nelements

             qd(:,k) = litter(:,imetabolic,m,l,k) * fd(:)

          END DO

          litter(:,imetabolic,m,l,:) = litter(:,imetabolic,m,l,:) - qd(:,:)
          n_mineralisation(:,m) = n_mineralisation(:,m) + qd(:,initrogen)
          !! 5.2.2 decompose same fraction of metabolic part of dead leaves.
          !  to avoid a multiple (for ibelow and iabove) modification of dead_leaves,
          !  we do this test to do this calcul only ones in 1,nlev loop
          if (l == iabove)  dead_leaves(:,m,imetabolic) = dead_leaves(:,m,imetabolic) * ( 1. - fd(:) )

          !! 5.2.3 put decomposed litter into active (or surface) pool + respiration
          som_input(:,itarget,m,icarbon) = som_input(:,itarget,m,icarbon) + &
               frac_soil(imetabolic,itarget,l) * qd(:,icarbon) / dt

          !BE CAREFUL: Here resp_hetero_litter is divided by dt to have a value which corresponds to
          ! the sechiba time step but then in stomate.f90 resp_hetero_litter is multiplied by dt.
          ! Perhaps it could be simplified. Moreover, we must totally adapt the routines to the dtradia/one_day
          ! time step and avoid some constructions that could create bug during future developments.
          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
               ( 1. - frac_soil(imetabolic,itarget,l) ) * qd(:,icarbon) / dt

          !! 5.3 woody litter: goes into active and slow carbon pools + respiration

          !! 5.3.1 total quantity of woody litter which is decomposed

          fd(:) = dt*litter_turn(iwoody) * &
               control_temp(:,l) * control_moist(:,l) * EXP( -3. * lignin_wood(:,m,l) )

          DO k = 1,nelements

             qd(:,k) = litter(:,iwoody,m,l,k) * fd(:)

          END DO

          litter(:,iwoody,m,l,:) = litter(:,iwoody,m,l,:) - qd(:,:)
          n_mineralisation(:,m) = n_mineralisation(:,m) + qd(:,initrogen)

          !! 5.3.2 non-lignin fraction of woody litter goes into
          !!       active/structural carbon pool + respiration (per time unit)

          som_input(:,itarget,m,icarbon) = som_input(:,itarget,m,icarbon) + &
               frac_soil(istructural,itarget,l) * qd(:,icarbon) * ( 1. - lignin_wood(:,m,l) ) / dt

          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
               ( 1. - frac_soil(istructural,itarget,l) ) * qd(:,icarbon) * &
               ( 1. - lignin_wood(:,m,l) ) / dt

          !! 5.3.3 lignin fraction of woody litter goes into
          !!       slow carbon pool + respiration (per time unit)

          som_input(:,islow,m,icarbon) = som_input(:,islow,m,icarbon) + &
               frac_soil(istructural,islow,l) * qd(:,icarbon) * lignin_wood(:,m,l) / dt

          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
               ( 1. - frac_soil(istructural,islow,l) ) * qd(:,icarbon) * lignin_wood(:,m,l) / dt


       ENDDO
    ENDDO

    som_input(:,iactive,:,initrogen) = som_input(:,iactive,:,icarbon)/CN_target(:,:,iactive)
    som_input(:,islow,:,initrogen) = som_input(:,islow,:,icarbon)/CN_target(:,:,islow)
    som_input(:,isurface,:,initrogen) = som_input(:,isurface,:,icarbon)/CN_target(:,:,isurface)

    n_mineralisation(:,:) = n_mineralisation(:,:) - &
         ( som_input(:,iactive,:,initrogen) + &
         som_input(:,islow,:,initrogen)   + &
         som_input(:,isurface,:,initrogen))*dt !! multiply by dt !!


    DO m=1,nvm
       summ(:,:)=zero
       DO l = 1, nlevs
          DO k = 1,nlitt
             summ(:,:)=summ(:,:)+litter(:,k,m,l,:)
          ENDDO
       ENDDO
       WHERE(summ(:,icarbon).LE.min_stomate)
          resp_hetero_litter(:,m) = resp_hetero_litter(:,m) + &
               summ(:,icarbon)
          n_mineralisation(:,m) = n_mineralisation(:,m) + &
               summ(:,initrogen)
          litter(:,imetabolic,m,iabove,icarbon) = zero
          litter(:,istructural,m,iabove,icarbon) = zero
          litter(:,iwoody,m,iabove,icarbon) = zero
          litter(:,imetabolic,m,ibelow,icarbon) = zero
          litter(:,istructural,m,ibelow,icarbon) = zero
          litter(:,iwoody,m,ibelow,icarbon) = zero
          litter(:,imetabolic,m,iabove,initrogen) = zero
          litter(:,istructural,m,iabove,initrogen) = zero
          litter(:,iwoody,m,iabove,initrogen) = zero
          litter(:,imetabolic,m,ibelow,initrogen) = zero
          litter(:,istructural,m,ibelow,initrogen) = zero
          litter(:,iwoody,m,ibelow,initrogen) = zero
       ENDWHERE
    ENDDO

  !! 6. calculate fraction of total soil covered by dead leaves

    CALL deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover)

  !! 7. (Quasi-)Analytical Spin-up : Start filling MatrixA

    IF (spinup_analytic) THEN

       MatrixA(:,:,:,:) = zero
       VectorB(:,:,:) = zero


       DO m = 1,nvm

          !- MatrixA : carbon fluxes leaving the litter

          MatrixA(:,m,istructural_above,istructural_above)= - dt*litter_turn(istructural) * &
               control_temp(:,iabove) * control_moist(:,iabove) * exp( -litter_struct_coef * lignin_struc(:,m,iabove) )

          MatrixA(:,m,istructural_below,istructural_below) = - dt*litter_turn(istructural) * &
               control_temp(:,ibelow) * control_moist(:,ibelow) * exp( -litter_struct_coef * lignin_struc(:,m,ibelow) )

          MatrixA(:,m,imetabolic_above,imetabolic_above) = - dt*litter_turn(imetabolic) * &
               control_temp(:,iabove) * control_moist(:,iabove)

          MatrixA(:,m,imetabolic_below,imetabolic_below) = - dt*litter_turn(imetabolic) * &
               control_temp(:,ibelow) * control_moist(:,ibelow)

          ! Flux leaving the woody above litter pool :
          MatrixA(:, m, iwoody_above, iwoody_above) = - dt * litter_turn(iwoody) * control_temp(:,iabove) * &
               control_moist(:,iabove) * exp( -3. * lignin_wood(:,m,iabove) )

          ! Flux leaving the woody below litter pool :
          MatrixA(:, m, iwoody_below, iwoody_below) = - dt * litter_turn(iwoody) * control_temp(:,ibelow) * &
               control_moist(:,ibelow) * exp( -3. * lignin_wood(:,m,ibelow))

          ! Flux received by the carbon surface from the woody above litter pool :
          MatrixA(:, m, isurface_pool, iwoody_above) = frac_soil(istructural, isurface, iabove) * &
               dt *litter_turn(iwoody) * &
               control_temp(:,iabove) * &
               control_moist(:,iabove) *  &
               exp( -3. * lignin_wood(:,m,iabove) ) * ( 1. -  lignin_wood(:,m,iabove) )

          ! Flux received by the carbon active from the woody below litter pool :
          MatrixA(:, m, iactive_pool, iwoody_below) = frac_soil(istructural, iactive, ibelow) * &
               dt *litter_turn(iwoody) * &
               control_temp(:,ibelow) * &
               control_moist(:,ibelow) * &
               exp( -3. * lignin_wood(:,m,ibelow) ) * ( 1. -  lignin_wood(:,m,ibelow) )

          ! Flux received by the carbon slow from the woody above litter pool :
          MatrixA(:, m, islow_pool, iwoody_above) = frac_soil(istructural, islow, iabove) * &
               dt *litter_turn(iwoody) * &
               control_temp(:,iabove) * &
               control_moist(:,iabove) * &
               exp( -3. * lignin_wood(:,m,iabove) ) * lignin_wood(:,m,iabove)

          ! Flux received by the carbon slow from the woody below litter pool :
          MatrixA(:, m, islow_pool, iwoody_below) =  frac_soil(istructural, islow, ibelow) * &
               dt *litter_turn(iwoody) * &
               control_temp(:,ibelow) * &
               control_moist(:,ibelow) * &
               exp( -3. * lignin_wood(:,m,ibelow) ) * lignin_wood(:,m,ibelow)


          !- MatrixA : carbon fluxes between the litter and the pools (the rest of the matrix is filled in stomate_soilcarbon.f90)
          MatrixA(:,m,isurface_pool,istructural_above) = frac_soil(istructural,isurface,iabove) * &
               dt*litter_turn(istructural) * &
               control_temp(:,iabove) * control_moist(:,iabove) * &
               exp( -litter_struct_coef * lignin_struc(:,m,iabove) ) * &
               ( 1. - lignin_struc(:,m,iabove) )


          MatrixA(:,m,iactive_pool,istructural_below) = frac_soil(istructural,iactive,ibelow) * &
               dt*litter_turn(istructural) * &
               control_temp(:,ibelow) * control_moist(:,ibelow) * &
               exp( -litter_struct_coef * lignin_struc(:,m,ibelow) ) * &
               ( 1. - lignin_struc(:,m,ibelow) )

          MatrixA(:,m,isurface_pool,imetabolic_above) =  frac_soil(imetabolic,isurface,iabove) * &
               dt*litter_turn(imetabolic) * control_temp(:,iabove) * control_moist(:,iabove)

          MatrixA(:,m,iactive_pool,imetabolic_below) =  frac_soil(imetabolic,iactive,ibelow) * &
               dt*litter_turn(imetabolic) * control_temp(:,ibelow) * control_moist(:,ibelow)

          MatrixA(:,m,islow_pool,istructural_above) = frac_soil(istructural,islow,iabove) * &
               dt*litter_turn(istructural) * &
               control_temp(:,iabove) * control_moist(:,iabove) * &
               exp( -litter_struct_coef * lignin_struc(:,m,iabove) )* &
               lignin_struc(:,m,iabove)


          MatrixA(:,m,islow_pool,istructural_below) = frac_soil(istructural,islow,ibelow) * &
               dt*litter_turn(istructural) * &
               control_temp(:,ibelow) * control_moist(:,ibelow) *  &
                  exp( -litter_struct_coef * lignin_struc(:,m,ibelow) )* &
                  lignin_struc(:,m,ibelow)


          !- VectorB : carbon input -

          VectorB(:,m,istructural_above) = litter_inc(:,istructural,m,iabove,icarbon)
          VectorB(:,m,istructural_below) = litter_inc(:,istructural,m,ibelow,icarbon)
          VectorB(:,m,imetabolic_above) = litter_inc(:,imetabolic,m,iabove,icarbon)
          VectorB(:,m,imetabolic_below) = litter_inc(:,imetabolic,m,ibelow,icarbon)
          VectorB(:,m,iwoody_above) = litter_inc(:,iwoody,m,iabove,icarbon)
          VectorB(:,m,iwoody_below) = litter_inc(:,iwoody,m,ibelow,icarbon)

          IF (printlev>=4) WRITE(numout,*) 'We filled MatrixA and VectorB'

          WHERE(litter(:,istructural,m,iabove,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,istructural_above) = &
                  ( CN_som_litter_longterm(:,m,istructural_above) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,m,iabove,icarbon)/litter(:,istructural,m,iabove,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,istructural,m,ibelow,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,istructural_below) = &
                  ( CN_som_litter_longterm(:,m,istructural_below) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,m,ibelow,icarbon)/litter(:,istructural,m,ibelow,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,imetabolic,m,iabove,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,imetabolic_above) = &
                  ( CN_som_litter_longterm(:,m,imetabolic_above) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,m,iabove,icarbon)/litter(:,imetabolic,m,iabove,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,imetabolic,m,ibelow,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,imetabolic_below) = &
                  ( CN_som_litter_longterm(:,m,imetabolic_below) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,m,ibelow,icarbon)/litter(:,imetabolic,m,ibelow,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,iwoody,m,iabove,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,iwoody_above) = ( CN_som_litter_longterm(:,m,iwoody_above) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,m,iabove,icarbon)/litter(:,iwoody,m,iabove,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,iwoody,m,ibelow,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,iwoody_below) = ( CN_som_litter_longterm(:,m,iwoody_below) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,m,ibelow,icarbon)/litter(:,iwoody,m,ibelow,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE




       ENDDO ! Loop over # PFTs


    ENDIF ! spinup analytic

    IF (printlev>=4) WRITE(numout,*) 'Leaving littercalc'

  END SUBROUTINE littercalc


!! ==============================================================================================================================\n
!! SUBROUTINE   : deadleaf
!!
!>\BRIEF        This routine calculates the deadleafcover.
!!
!! DESCRIPTION  : It first calculates the lai corresponding to the dead leaves (LAI) using
!! the dead leaves carbon content (DL) the specific leaf area (sla) and the
!! maximal coverage fraction of a PFT (vegetmax) using the following equations:
!! \latexonly
!! \input{deadleaf1.tex}
!! \endlatexonly
!! \n
!! Then, the dead leaf cover (DLC) is calculated as following:\n
!! \latexonly
!! \input{deadleaf2.tex}
!! \endlatexonly
!! \n
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE: ::deadleaf_cover
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                          :: npts           !! Domain size - number of grid pixels (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(in)  :: dead_leaves    !! Dead leaves per ground unit area, per PFT,
                                                                          !! metabolic and structural
                                                                          !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)          :: veget_cov_max  !! PFT "Maximal" coverage fraction of a PFT defined in
                                                                          !! the input vegetation map
                                                                          !! @tex $(m^2 m^{-2})$ @endtex

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts), INTENT(out)           :: deadleaf_cover !! Fraction of soil covered by dead leaves over all PFTs
                                                                          !! (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts)                        :: dead_lai       !! LAI of dead leaves @tex $(m^2 m^{-2})$ @endtex
    INTEGER(i_std)                                      :: j              !! Index (unitless)
    REAL(r_std), DIMENSION(npts)                        :: total_dead_leaves !! imetabolic + istructural
!_ ================================================================================================================================

  !! 1. LAI of dead leaves

    dead_lai(:) = zero

    DO j = 1,nvm !Loop over PFTs
       total_dead_leaves(:) = dead_leaves(:,j,imetabolic) + dead_leaves(:,j,istructural)
       dead_lai(:) = dead_lai(:) + biomass_to_lai( total_dead_leaves, npts, j) &
            * veget_cov_max(:,j)
    ENDDO

  !! 2. fraction of soil covered by dead leaves

    deadleaf_cover(:) = un - exp( - 0.5 * dead_lai(:) )

    IF (printlev>=4) WRITE(numout,*) 'Leaving deadleaf'

  END SUBROUTINE deadleaf


!! ================================================================================================================================
!! FUNCTION     : control_moist_func
!!
!>\BRIEF        Calculate moisture control for litter and soil C decomposition
!!
!! DESCRIPTION  : Calculate moisture control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_moist_func1.tex}
!! \endlatexonly
!! \n
!! with M the moisture control factor and soilmoisutre, the soil moisture
!! calculated in sechiba.
!! Then, the function is ranged between Moistcont_min and 1:\n
!! \latexonly
!! \input{control_moist_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE : ::moistfunc_result
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  FUNCTION control_moist_func (npts, moist_in) RESULT (moistfunc_result)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)               :: npts                !! Domain size - number of grid pixel (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in) :: moist_in            !! relative humidity (unitless)

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts)             :: moistfunc_result    !! Moisture control factor (0.25-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    moistfunc_result(:) = -moist_coeff(1) * moist_in(:) * moist_in(:) + moist_coeff(2)* moist_in(:) - moist_coeff(3)
    moistfunc_result(:) = MAX( moistcont_min, MIN( un, moistfunc_result(:) ) )

  END FUNCTION control_moist_func


!! ================================================================================================================================
!! FUNCTION     : control_temp_func
!!
!>\BRIEF        Calculate temperature control for litter and soild C decomposition
!!
!! DESCRIPTION  : Calculate temperature control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_temp_func1.tex}
!! \endlatexonly
!! \n
!! with T the temperature control factor, temp the temperature in Kelvin of
!! the air (for aboveground litter) or of the soil (for belowground litter
!! and soil)
!! Then, the function is limited in its maximal range to 1:\n
!! \latexonly
!! \input{control_temp_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE: ::tempfunc_result
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  FUNCTION control_temp_func (npts, temp_in) RESULT (tempfunc_result)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfunc_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    tempfunc_result(:) = exp( soil_Q10 * ( temp_in(:) - (ZeroCelsius+tsoil_ref)) / Q10 )
    tempfunc_result(:) = MIN( un, tempfunc_result(:) )

  END FUNCTION control_temp_func

END MODULE stomate_litter
